# telemetry-javascript

### Telemetry Client Library Wrapper for Javascript

Consists of testing, linting, building and deploying of the package.

### Usage

When setting up the continuous integration two environment variables are necessary: `NPM_TOKEN`, `GH_TOKEN`. 

### Language

The package uses [Typescript](https://www.typescriptlang.org/) because the consumers get hints on how to use the library within the sourcecode.

### Development environment setup
* Ensure your `python` command is pointed to Python `2.7.x`. The `unix-dgram` package introduces this dependency on Python.
    * [Install node-gyp](https://stackoverflow.com/questions/21365714/nodejs-error-installing-with-npm)
* Install the package dependencies by running `npm install`.

### Testing

[Jest](https://jestjs.io/) does the testing as it's not only a test-runner, but also includes assertions and a virtual DOM library.

```bash
npm run test # Single test run, suitable for continuous integration
npm run test:watch # Reruns modified tests on modification, good for development
npm run test:coverage # Single test run with code coverage metrics
```

### Linting

Linting is done with [ESlint](https://eslint.org/) and [Prettier](https://prettier.io/) because TSlint is [no longer maintained](https://blog.palantir.com/tslint-in-2019-1a144c2317a9).

```bash
npm run lint # Checks for lint failures based on ESlint rules and Prettier config
npm run lint:fix # Fixes lint errors where possible
```

### Bundling

Besides the basic Typescript transpilation, the project uses [microbundle](https://github.com/developit/microbundle) 
to generate separate optimized files for UMD, module and Node bundles.

```bash
npm run bundle # Generates the optimized files into the dist directory
```

### Release

[Semantic release](https://github.com/semantic-release/semantic-release) is used to automatically increment package versioning based on commit message. The package will be released when a PR is merged to `master` with commits which contain semantic-release-style commit messages.

### Installation

To use the package, run the following command in your application folder.

```bash
npm install @emartech/telemetry-javascript
```
